****************************************************************************
****************************************************************************
**                                                               
**  Platinum Edition - Data Structure Definitons
**
**  Copyright 2005 by Patrick Davidson.  This software may be freely
**  modified and/or copied with no restrictions.  There is no warranty.
**
**  by Patrick Davidson (pad@calc.org, http://pad.calc.org/)
**
**  Last updated September 1, 2005
**
****************************************************************************
****************************************************************************

******************************************** BULLET STRUCTURE

b_type  equ     0                       ;word offset into jump table
b_dmg   equ     2                       ;number of damage points
b_x     equ     4                       ;X-coordinate
b_w     equ     6                       ;width
b_y     equ     8                       ;Y-coordinate
b_h     equ     10                      ;height
b_image equ     12                      ;offset into image table
b_data  equ     14                      ;2 bytes for other data
b_size  equ     16      

******************************************** ENEMY BULLET STRUCTURE

eb_type equ     0                       ;offset into jump table
eb_dmg  equ     2                       ;number of damage points
eb_x    equ     4                       ;X-coordinate
eb_w    equ     6                       ;width
eb_y    equ     8                       ;Y-coordinate
eb_h    equ     10                      ;height
eb_image equ    12                      ;offset into image table
eb_data equ     14                      ;2 bytes for other data
eb_size equ     16      

******************************************** ENEMY DATA STRUCTURE

num_en  equ     26      

e_type  equ     0                       ;offset into enemy table
e_dmg   equ     2                       ;strength of the enemy
e_x     equ     4                       ;X-coordinate
e_w     equ     6                       ;width
e_y     equ     8                       ;Y-coordinate
e_h     equ     10                      ;height
e_image equ     12                      ;offset into image table
e_data  equ     14                      ;2 bytes for other data
e_dstry equ     16                      ;destruction code offset
e_misc  equ     18                      ;more data
e_path  equ     20                      ;offset into path data
e_timer equ     22                      ;timer
e_size  equ     24     

******************************************** MAIN VARIABLES
*
* This defines the game's main variables.  The 'rs' macro reserves the
* specified amount of space by incrementing the offset counter.  Overall,
* this section serves to calculate the offsets of each variable.  This
* system allows variables to be easily added, removed, or resized, as the
* offsets themselves are not hard-coded.  Near the bottom, data_end is
* defined with a size of zero to set that equal to the length of the data
* so far.  At the end, 'offset' is the total size of all this data.
*
********

SCREEN_SIZE     set     32*106

offset  set     0       

rs      MACRO   
\1      set     offset  
offset  set     offset+\2       
        ENDM    

        rs      s,SCREEN_SIZE           ; Screen buffer
        rs      s2,SCREEN_SIZE

        rs      data_start,0            ; Start of data to be saved

        rs      score,4                 ; Score (BCD)

        rs      pattern_type,2          ; Type of movement pattern
        rs      pattern_stage,2         ; Segment of pattern
        rs      pattern_x,2             ; Pattern X offset
        rs      pattern_y,2             ; Pattern Y offset
        rs      level_timer,2           ; Number of cycles in this level

        rs      bullets_data,b_size*24  ; All data for player bullets
        rs      enemy_bullets,eb_size*16        ; All data for enemy bullets
        rs      enemies_data,e_size*26  ; All data for enemies
        rs      enemies_data_end,0      

        rs      bpos,2  
        rs      cash,4                  ; Amount of credits
        rs      num_b,2                 ; Number of player's bullets
        rs      fire_delay,2            ; -(Time Between Shots)

        rs      player_ship,2           ; Player's ship (0, 1, 2)
        rs      player_dmg,2            ; Condition of player's ship
        rs      player_xc,2             ; Player X coordinate
        rs      player_width,2          ; Width of player's ship
        rs      player_yc,2             ; Player Y coordinate
        rs      player_height,2         ; Hieght of player's ship
        rs      player_image,2          ; Sprite for player's ship

        rs      player_weaponx,2        ; X-Offset of gun in ship
        rs      player_speed,2          ; Pixels per frame

        rs      weapon_selected,2       ; Selected weapon (*2)
        rs      lock_status,2           ; Count-up to fire ready

        rs      cycle_counter,4         ; Incremented every frame
        rs      time_bonus,4            ; Bonus points for quick finish

        rs      enemies_remaining,2     ; Number of enemies left

        rs      level_number,2          ; Level number
        rs      xy_pointers,100         ; Coordinate table

        rs      weapons_available,1     ; Bit mask for avail. weapons
        rs      game_finished,1         ; Game has been finished
        rs      cheated,1               ; Set if player has cheated
        rs      flags,1                 ; Flags for miscellaneous use
        rs      slow_flag,1             ; set if game running too slow
        rs      game_started,1
        rs      game_lose,2

        rs      starfield_y,2
        rs      edge_y,2

        rs      special_weapon_delay,2  ; Special weapon delay
        rs      special_countdown,2     ; Countdown to second shot

        rs      speed,2

        rs      clear_end,0             ; End of data cleared for new game

        rs      back_type,2
        rs      ludicrous,2
        rs      difficulty,2            ; Difficulty level (1-3)
        rs      classic,2
        rs      seed,2
        
        rs      data_end,0              ; End of data to be saved

        rs      plane0,4
        rs      plane1,4

        rs      key_data,10
        rs      old_keys,10
        rs      key_edge,10

        rs      _string_buffer,40       ; String buffer (for sprintf)
        rs      bcd_buffer,4

*************************************** HARDWARE KEY READING
*
* This macro tests a key, using data from the last call to Read_All_Keys.
* Its syntax is
*
* GETKEY row89,column89,row92,column92
*
* The zero flag is set if it is pressed, and clear otherwise.
*
* A5 must be pointing to the game's data structures.
*
********

GETKEY  MACRO
    IFD ti89
        btst    #\2,key_data+\1(a5)
    ENDIF
    IFND ti89
        btst    #\4,key_data+\3(a5)
    ENDIF
        ENDM

GETEDGE  MACRO
    IFD ti89
        btst    #\2,key_edge+\1(a5)
    ENDIF
    IFND ti89
        btst    #\4,key_edge+\3(a5)
    ENDIF
        ENDM

*************************************** ROM CALLING FUNCTIONS

    IFND ti92
JMP_ROM MACRO
        move.l  (200).w,a0
        move.l  \1*4(a0),a0
        jmp     (a0)
        ENDM

JSR_ROM MACRO
        move.l  (200).w,a0
        move.l  \1*4(a0),a0
        jsr     (a0)
        ENDM
    ENDIF

    IFD ti92
tios::DrawStr equ tios::DrawStrXY
tios::HeapAllocHigh equ tios::HeapAlloc

JMP_ROM MACRO
        jmp     tios::\1
        ENDM

JSR_ROM MACRO
        jsr     tios::\1
        ENDM
    ENDIF
